use leptos::*;
use leptos_router::*;
use serde::{Deserialize, Serialize};

// use crate::i18n::*;

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct WatermarkedImage {
    title: String,
    slug: String,
    #[serde(default)]
    description: String,
    image_without_watermark: contentful::models::Asset,
    image_with_watermark: contentful::models::Asset
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Gallery {
    title: String,
    slug: String,
    #[serde(default)]
    description: String,
    images: Vec<WatermarkedImage>
}

#[server(GetGallery, "/api", "GetJson")]
pub async fn get_gallery() -> Result<Gallery, ServerFnError> {
    use crate::services::contentful;
    let contentful_client = contentful::get_contentful_client();
    let slug = "photography-portfolio";
    let query_string = format!("?content_type=imageGallery&include=10&fields.slug={}", slug);
    let galleries = contentful_client.get_entries_by_query_string::<Gallery>(&query_string).await;
    match galleries {
        Ok(found_galleries) => {
            if found_galleries.len() != 1 {
                return Err(ServerFnError::ServerError("Found none or more than one".to_string()));
            }
            Ok(found_galleries[0].clone())
        },
        Err(e) => Err(ServerFnError::ServerError(e.to_string())),
    }
}

#[derive(Params, PartialEq)]
struct GalleryEntryParams {
    slug: String
}

#[component]
pub fn GalleryEntry() -> impl IntoView {
    let params = use_params::<GalleryEntryParams>();
    let slug= move || {
        params.with(|params| params.as_ref()
            .map(|params| params.slug.clone())
            .unwrap_or_default())
    };
    let gallery = create_resource(|| (), |_| async move { get_gallery().await });

    view! {
        <Suspense fallback=move || view! { <p>"Loading..."</p> }>
            {move || {
                gallery.get().map(|gallery_loaded|
                    match gallery_loaded {
                        Ok(gallery) => {
                            let image = move || gallery.images.into_iter()
                                .find(move |image| image.slug == slug());
                            match image() {
                                None => view! { <div/> },
                                Some(image) => {
                                    let display_url = format!("https:{}?w=2400&h=2400&q=80&fm=avif", &image.image_with_watermark.file.url);
                                    view! {
                                        <div class="open-entry">
                                            <img src=display_url alt=image.title.clone()/>
                                            <h3>{image.title.clone()}</h3>
                                            <p>{image.description}</p>
                                        </div>
                                    }
                                }
                            }
                        },
                        Err(error) => view! { <div>"Error: "{error.to_string()}</div> }
                    }
                )

            }}
        </Suspense>
    }

}

#[component]
pub fn Gallery() -> impl IntoView {
    let gallery = create_resource(|| (), |_| async move { get_gallery().await });

    view! {
        <main class="main-width gallery-wrapper">
            <Outlet/>
            <Suspense fallback=move || view! { <p>"Loading..."</p> }>
                {move || {
                    gallery.get().map(|gallery_loaded|
                        match gallery_loaded {
                            Ok(gallery) => {
                                view! {
                                    <div class="gallery">
                                        <h3>{gallery.title}</h3>
                                        <p>{gallery.description}</p>
                                        <ul>
                                            {gallery.images.into_iter()
                                                .map(|image| {
                                                    let thumbnail_url = format!("https:{}?w=800&h=800&q=80&fm=avif", &image.image_without_watermark.file.url);
                                                    let link_url = format!("/gallery/{}", &image.slug);
                                                    view! { <li><A href=link_url><img src=thumbnail_url alt=image.title/></A></li> }
                                                }).collect::<Vec<_>>()
                                            }
                                        </ul>
                                    </div>
                                }
                            },
                            Err(error) => view! { <div>"Error: "{error.to_string()}</div> }
                        }
                    )

                }}
            </Suspense>
        </main>
    }
}
